#
# spec file for package lxcfs
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


# On pre-15 SLE versions, _sharedstatedir was /usr/com -- which is just wrong.
%if 0%{?suse_version} < 1500
%define _sharedstatedir /var/lib
%endif

Name:           lxcfs
Version:        5.0.4
Release:        1.3
Summary:        FUSE filesystem for LXC
License:        Apache-2.0
Group:          System/Management
URL:            https://linuxcontainers.org/lxcfs
Source:         https://linuxcontainers.org/downloads/%{name}/%{name}-%{version}.tar.gz
Source1:        https://linuxcontainers.org/downloads/%{name}/%{name}-%{version}.tar.gz.asc
Source2:        %{name}.keyring
BuildRequires:  fdupes
BuildRequires:  help2man
BuildRequires:  libtool
BuildRequires:  meson >= 0.50
BuildRequires:  pam-devel
BuildRequires:  pkg-config
BuildRequires:  python3
BuildRequires:  python3-Jinja2
BuildRequires:  pkgconfig(fuse)
BuildRequires:  pkgconfig(systemd)
%{?systemd_requires}

%description
LXCFS is a small FUSE filesystem written with the intention of making Linux
containers feel more like a virtual machine. It started as a side-project of
LXC but is usable by any runtime.

%package hooks-lxc
Summary:        LXC hooks for %{name}
Group:          System/Management
Requires:       %{name} = %{version}
%if 0%{?sle_version} < 150000
Supplements:    packageand(%{name}:liblxc1)
%else
Supplements:    (%{name} and liblxc1)
%endif
BuildArch:      noarch

%description hooks-lxc
Configuration to add hooks for %{name} so that it automatically interoperates
with LXC for all containers.

%prep
%autosetup -p1

%build
%meson \
	-D init-script=systemd \
	-D runtime-path=%{_rundir} \
	%{nil}
%meson_build

%install
# The shared library liblxcfs.so used by lxcfs is not supposed to be used by
# any other program. lxcfs will automatically install it to {_libdir}/{name}
# which is out of the way of any other users.
%meson_install
install -d -m 0755 %{buildroot}%{_sharedstatedir}/%{name}

# systemd service and sysv-init compat wrapper.
mkdir -p %{buildroot}%{_sbindir}
ln -s %{_sbindir}/service %{buildroot}%{_sbindir}/rc%{name}

# Clean up.
find %{buildroot} -type f -name '*.la' -delete
%fdupes %{buildroot}

%pre
%service_add_pre lxcfs.service

%post
%service_add_post lxcfs.service

%preun
%service_del_preun lxcfs.service

%postun
%service_del_postun lxcfs.service

%files
%defattr(-,root,root)
%doc AUTHORS README*
%license COPYING
%{_sbindir}/*
%{_bindir}/*
%{_datadir}/%{name}
%{_mandir}/man1/*
%{_unitdir}/*

# Mountpoint for lxcfs.
%dir %{_sharedstatedir}/%{name}

# The lxcfs executable requires liblxcfs.so be installed. It calls dlopen() to
# dynamically reload the shared library on upgrade. This is important. Do *not*
# split into a separate package and do not turn this into a versioned shared
# library! (This shared library allows lxcfs to be updated without having to
# restart it which is good when you have important system containers running!)
%dir %{_libdir}/%{name}
%{_libdir}/%{name}/liblxcfs.so

%files hooks-lxc
%defattr(-,root,root)
%{_datadir}/lxc

%changelog
* Mon Jul 31 2023 Dirk Müller <dmueller@suse.com>
- update to 5.0.4:
  * Fixed the output format of /proc/diskstats
  * Fixed an issue with cpuset path tracking when used with containerd
* Tue Feb 28 2023 Callum Farmer <gmbr3@opensuse.org>
- Correct BR from python3-jinja2 to python3-Jinja2, fixes
  dependency resolving on older distros
* Sat Oct 29 2022 Aleksa Sarai <asarai@suse.com>
- Update to LXCFS 5.0.2. The full upstream changelog is available from:
  https://discuss.linuxcontainers.org/t/lxcfs-5-0-lts-has-been-released/13535
  https://discuss.linuxcontainers.org/t/lxcfs-5-0-1-has-been-released/14709
  https://discuss.linuxcontainers.org/t/lxcfs-5-0-2-has-been-released/14811
  boo#1204842
  LXCFS 5.0.2:
  * Fix crash during load/reload on FUSE 3
  LXCFS 5.0.1:
  * Fixes to /sys/devices/system/cpu handling on FUSE 2
  * Fix for a formatting issue in /proc/stat
  * Fix for a cgroup2 issue in handling cgroups with no CPU limits
  * Fix for library reload logic
  LXCFS 5.0:
  * Switch to meson
  + Complete cgroup2 support
  + /proc/slabinfo support
  + /sys/devices/system/cpu support
- Drop no-longer needed patches.
  - include-fixes.patch
* Fri Sep 30 2022 Dirk Müller <dmueller@suse.com>
- add include-fixes.patch
- reenable FORTIFY_SOURCE=3
* Sun Sep 25 2022 Johannes Kastl <kastl@b1-systems.de>
- fix build by using FORTIFY_SOURCE=2
- fix Supplements syntax
* Tue Feb 22 2022 Johannes Kastl <kastl@b1-systems.de>
- update to 4.0.12:
  * CPU sum in /proc/stat is now preserved on cpuset changes
  * Expose an empty cgroup tree when host system is on a unified hierarchy
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-12-has-been-released/13287
* Tue Feb 22 2022 Johannes Kastl <kastl@b1-systems.de>
- update to 4.0.11:
  NOTE: This was originally released as LXCFS 4.0.10 but a bug caused the published release tarball to be missing a file. LXCFS 4.0.11 includes that fix.
  * Fixed a potential deadlock in the cpuinfo handler
  * Reduced daemon verbosity
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-11-has-been-released/12426
* Tue Feb 22 2022 Johannes Kastl <kastl@b1-systems.de>
- update to 4.0.9:
  * Improve libfuse2/libfuse3 handling
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-9-lts-has-been-released/11617
* Mon May 24 2021 Johannes Kastl <kastl@b1-systems.de>
- update to 4.0.8:
  * Correct file size for proc files
  * for full list of changes see
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-8-lts-has-been-released/10998
* Mon May 24 2021 Johannes Kastl <kastl@b1-systems.de>
- update to 4.0.7:
  * Consistent swap behavior (documented in README)
  * for full list of changes see
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-7-lts-has-been-released/9893
* Mon May 24 2021 Johannes Kastl <kastl@b1-systems.de>
- update to 4.0.6:
  * Add support for fuse3
  * Update for new diskstats fields
  * Fix some build issues
  * for full list of changes see
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-6-lts-has-been-released/9236
* Wed Aug 12 2020 Pavol Cupka <palica@liguros.net>
- update to LXCFS 4.0.5
  detailed changelog:
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-5-lts-has-been-released/8602
* Sat Jun 27 2020 Pavol Cupka <palica@liguros.net>
- update to LXCFS 4.0.4 (released on the 18th of June 2020)
  detailed changelog:
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-4-lts-has-been-released/8212
* Sun May 10 2020 Pavol Cupka <palica@liguros.net>
- update to LXCFS 4.0.3 (released on the 17th of April 2020)
  detailed changelog:
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-3-lts-has-been-released/7470
* Wed Apr  1 2020 Aleksa Sarai <asarai@suse.com>
- Update to LXCFS 4.0.1. The full upstream changelog is available from:
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-lts-has-been-released/7031
  https://discuss.linuxcontainers.org/t/lxcfs-4-0-1-lts-has-been-released/7130
  Related to the LXD 4.0.0 update (boo#1168338).
  + cgroup2: Support for the new unified cgroup hierarchy
  + /proc/cpuinfo and cpu output in /proc/stat based on cpu shares
  + /proc/loadavg virtualization
  + pidfd supported process tracking
  * Improved command line options
  * Compiler based hardening
  * Minimal compiler based resource management
- Clean up %%if macro.
* Mon Feb  3 2020 Dominique Leuenberger <dimstar@opensuse.org>
- BuildRequire pkgconfig(systemd) instead of systemd: allow OBS to
  shortcut through the -mini flavors.
* Thu Sep 19 2019 Pavol Cupka <palica@liguros.net>
- update to version 3.1.2 (released on the 24th of July 2019)
  New Features:
  * Add support for per-container cpu usage in /proc/stat
  * Add support for load average (loadavg) virtualization
  * Display cpus in /proc/cpuinfo based on cpu quotas
  * Allow to disable swap in /proc/meminfo output
  * Virtualize /sys/devices/system/cpu/online
  * Enable higher precision output in /proc/uptime
  * Add support for FUSE nonempty option
  Bugfixes:
  * bindings: ensure that opts is non NULL
  * Makefile: Fix typo in file name
  * remove unused functions
  * sys dirs do not need to implement 'read' method
  * lxcfs: coding style update
  * config: Adds RPM spec file.
  * config: Adds reload mode to sysvinit and systemd scripts.
  * bindings: prevent NULL pointer dereference
  * stat: check for out of bound access to cpuacct usage data
  * calc_hash(): do not apply modulo LOAD_SIZE
  * tests: include missing sys/sysmacros.h header
  * bindings: prevent double free
  * bindings: better logging for write_string()
  * meminfo: set ShmemHugePages and ShmemPmdMapped to zero
  * bindings: fix memory leak in calc_pid()
  * travis: fix .travis.yml
  * bindings: fix memory leak in proc_loadavg_read()
* Sun Jun  9 2019 Aleksa Sarai <asarai@suse.com>
- Make lxcfs-hooks-lxc supplement liblxc1 instead of lxc, since there are other
  liblxc1 users (such as LXD).
* Tue Mar 26 2019 Aleksa Sarai <asarai@suse.com>
- Update to version 3.0.3. The changelog is fairly significant from 2.x, but
  the main difference is tha the PAM module has been moved to the LXC project,
  and is no longer hosted in lxcfs.
- Add workaround for pre-15 distros, where _sharedstatedir was inexplicably
  /usr/com, to use the correct directory of /var/lib.
* Tue Mar 26 2019 Aleksa Sarai <asarai@suse.com>
- Rework packaging to be a more modern openSUSE-style.
* Wed Jun 13 2018 dcassany@suse.com
- Make use of %%license macro
* Tue Oct 31 2017 opensuse_buildservice@ojkastl.de
- update to version 2.0.8
  This is the eigth bugfix release for LXCFS 2.0. This includes the following bugfixes:
  * bindings: Add mountpoint for unified hierarchy
  * bindings: Calculate uptime via proc//stat
  * bindings: Revert virtualization of 'btime' field due to regressions in ps
  * doc: Update README
  * init: Add cgroupfs-mount to Should-Start/Stop sysvinit LSB headers
  * lib: Add common fallback dlopen for liblxcfs.so
  * lib: Fix the installation directory for liblxcfs to ${libdir}/lxcfs
  * pam: Add a 'all' option for -c
  * pam: Chown cgroup.procs file on unified hierarchy
  * pam: Report back when we find a unified hierarchy
  * tests: Fix invalid comparison
  * uptime: Fix a problem with subsequent reads
* Wed May 17 2017 opensuse_buildservice@ojkastl.de
- update to version 2.0.7
  This is the seventh bugfix release for LXCFS 2.0. This includes the following bugfixes:
  * Remove unused variable
  * Also check next variable for NULL on cg_rmdir
  * virtualize the 'btime' field of /proc/stat
  * cleanup: return false instead of NULL as bool
  * Limit memswlimit by TotalSwap
  * pam_cgfs: remove dead assignment
  * pam_cgfs: return created directly
  * pam_cgfs: make sure that **p is not NULL
  * bindings: Want space for ints? Call sizeof(int)!
  * pam_cgfs: make trim() safer
  * pam_cgfs: error out on failure in cgv2_init()
  * pam_cgfs: remove dead assignment
  * bindings: implement guest nice
  * bindings: increase reserved buffer size a little
- added signature verification
* Fri Jan 27 2017 opensuse_buildservice@ojkastl.de
- deleted patches 0001 to 00024, as they are upstream already
  - 0001-Fix-test_reload-for-lxcfs-chroot.patch
  - 0002-Virtualize-more-of-the-meminfo-fields.patch
  - 0003-pam-fix-race-in-cgroup-creation.patch
  - 0004-meminfo-don-t-show-negative-swapfree.patch
  - 0005-bindings-improve-debugging.patch
  - 0006-bindings-use-openat-fd-for-fstatat-unlinkat.patch
  - 0007-bindings-close-open-fds-on-error.patch
  - 0008-bindings-grant-access-to-var-lib-lxcfs.patch
  - 0009-bindings-enable-access-to-var-lib-lxcfs-cgroup.patch
  - 0010-bindings-allow-access-to-var-lib-lxcfs-proc.patch
  - 0011-lxcfs-bindings-show-.-and-.-dir-entries.patch
  - 0012-lxcfs-better-fs-behavior-on-var-lib-lxcfs.patch
  - 0013-bindings-non-functional-changes.patch
  - 0014-bindings-set-errno-in-pick_controller_from_path.patch
  - 0015-bindings-more-consistent-fs-behavior.patch
  - 0016-add-pld-linux-support.patch
  - 0017-don-t-use-argv-0-in-usage-output.patch
  - 0018-bindings-revert-cgroup-check.patch
  - 0019-bindings-improve-returned-errnos.patch
  - 0020-bindings-make-rmdir-behave-more-consistently.patch
  - 0021-libtool-do-not-link-lxcfs-against-liblxcfs.patch
  - 0022-bindings-lxcfs-improve-debugging.patch
  - 0023-bindings-fix-debug-macro.patch
  - 0024-bindings-restore-original-working-directory.patch
* Wed Jan 25 2017 opensuse_buildservice@ojkastl.de
- update to version 2.0.6
  This is the sixth bugfix release for LXCFS 2.0. This includes the following bugfixes:
    Fix swap values with nested cgroups
    tests: Fix run on ppc64el
    Fix wrong scanning of memory.stat
- update to version 2.0.5
  This is the fifth bugfix release for LXCFS 2.0. This includes the following bugfixes:
    Add Documentation key to systemd unit
    bindings: allow getattr on O_WRONLY files
    bindings: remove noop check
    fix Active/Inactive /proc/meminfo
    macro: add header for shared macros
    pam_cgfs: reimplement and add cgroupfs v2 support
    pam_cgfs: re-use cgroups that already belong to us
    pam_cgfs: handle cgroupfs v1 cpuset controller
    pam_cgfs: improve logging
    cgroups: handle non-existent isolcpus file
- update to version 2.0.4
  This is the fourth bugfix release for LXCFS 2.0. This includes the following bugfixes:
  Fix test_reload for lxcfs chroot
  Virtualize more of the meminfo fields
  pam: fix race in cgroup creation
  meminfo: don't show negative swapfree
  bindings: improve debugging
  bindings: use openat fd for fstatat(), unlinkat()
  bindings: close open fds on error
  bindings: grant access to /var/lib/lxcfs
  bindings: enable access to /var/lib/lxcfs/cgroup
  bindings: allow access to /var/lib/lxcfs/proc
  lxcfs, bindings: show "." and ".." dir entries
  lxcfs: better fs behavior on /var/lib/lxcfs
  bindings: non functional changes
  bindings: set errno in pick_controller_from_path()
  bindings: more consistent fs behavior
  add pld linux support
  don't use argv[0] in usage output
  bindings: revert cgroup check
  bindings: improve returned errnos
  bindings: make rmdir behave more consistently
  libtool: do not link lxcfs against liblxcfs
  bindings, lxcfs: improve debugging
  bindings: fix debug macro
  autotools: add -avoid-version
  bindings: restore original working directory
  bindings: add function to check fs type
  bindings: agnostic naming
  bindings: use chroot() on ramfs
  bindings: fix type weirdness with statfs f_type
  bindings: make pivot_enter() contain all its code
* Wed Aug 31 2016 cbrauner@suse.de
- update lxcfs to 2.0.3
- add 0001-Fix-test_reload-for-lxcfs-chroot.patch
- add 0002-Virtualize-more-of-the-meminfo-fields.patch
- add 0003-pam-fix-race-in-cgroup-creation.patch
- add 0004-meminfo-don-t-show-negative-swapfree.patch
- add 0005-bindings-improve-debugging.patch
- add 0006-bindings-use-openat-fd-for-fstatat-unlinkat.patch
- add 0007-bindings-close-open-fds-on-error.patch
- add 0008-bindings-grant-access-to-var-lib-lxcfs.patch
- add 0009-bindings-enable-access-to-var-lib-lxcfs-cgroup.patch
- add 0010-bindings-allow-access-to-var-lib-lxcfs-proc.patch
- add 0011-lxcfs-bindings-show-.-and-.-dir-entries.patch
- add 0012-lxcfs-better-fs-behavior-on-var-lib-lxcfs.patch
- add 0013-bindings-non-functional-changes.patch
- add 0014-bindings-set-errno-in-pick_controller_from_path.patch
- add 0015-bindings-more-consistent-fs-behavior.patch
- add 0016-add-pld-linux-support.patch
- add 0017-don-t-use-argv-0-in-usage-output.patch
- add 0018-bindings-revert-cgroup-check.patch
- add 0019-bindings-improve-returned-errnos.patch
- add 0020-bindings-make-rmdir-behave-more-consistently.patch
- add 0021-libtool-do-not-link-lxcfs-against-liblxcfs.patch
- add 0022-bindings-lxcfs-improve-debugging.patch
- add 0023-bindings-fix-debug-macro.patch
- add 0024-bindings-restore-original-working-directory.patch
* Wed Aug 31 2016 cbrauner@suse.de
- remove 0001-skip-empty-entries-under-proc-self-cgroup.patch
* Sat Jul 23 2016 cbrauner@suse.com
- remove explicit Provides for pam_cgfs subpackage
* Mon Jul 11 2016 cbrauner@suse.de
- install pam_cgfs.so with the pam_cgfs subpackage
* Sun Jul 10 2016 cbrauner@suse.com
- add 0001-skip-empty-entries-under-proc-self-cgroup.patch
* Thu Jul  7 2016 cbrauner@suse.de
- Remove lxcfs-devel.
- Install shared library liblxcfs.so into /usr/lib/lxcfs as it is not supposed
  to be used by any other program.
* Thu Jul  7 2016 cbrauner@suse.de
- rename devel package from liblxcfs-devel to lxcfs-devel
* Thu Jul  7 2016 cbrauner@suse.de
- make sure that lxcfs install the shared library liblxcfs.so:
  The lxcfs executable requires liblxcfs.so be installed. It calls dlopen() to
  dynamically reload the shared library on upgrade. This is important. Do *not*
  split into a separate package and do not turn this into a versioned shared
  library! (This shared library allows lxcfs to be updated without having to
  restart running containers!)
* Thu Jul  7 2016 cbrauner@suse.de
- add lxcfs package
